package Character;

use strict;
no strict 'refs';
use base 'Noun';


sub new  {
        my $class = shift;
        my $id = shift;

        my $self = $class->SUPER::new($id);

        bless ($self, $class);   

        $self->{sightings} = ();

        return $self;
}


sub add_saying
{
 my $self = shift;
 my $txt = shift;

 push(@{$self->{attr}->{sayings}},$txt);
}


sub get_saying
{
 my $self = shift;
 
 my $r = rand @{$self->{attr}->{sayings}};

 return(${$self->{attr}->{sayings}}[$r]);
}

sub has_saying
{
 my $self = shift;

 if (@{$self->{attr}->{sayings}})
 {
  if (defined $self->{attr}->{saying_chance})
  {
   if (rand(100) < $self->{attr}->{saying_chance})
   {
    return(1);
   }
  }
  else
  {
   return(1);
  }
 }
 return(0);
}


sub sees
{
 my $self = shift;
 my $seen = shift;

 if (defined $self->{sightings}->{$seen->id()})
 {
  $self->{sightings}->{$seen->id()}++;
 }
 else
 {
  $self->{sightings}->{$seen->id()} = 1;
 }

}



sub sightings
{
 my $self = shift;
 my $seen = shift;

 if (defined $self->{sightings}->{$seen->id()})
 {
  return($self->{sightings}->{$seen->id()})
 }
 
 return(0);
}


sub forget
{
 my $self = shift;
 my $seen = shift;

 if (defined $self->{sightings}->{$seen->id()})
 {
  $self->{sightings}->{$seen->id()} = 0;
 }

}


sub tick
{
 my $self = shift;
 my $actor = shift;

 $self->SUPER::tick($self);

 if ($self->mobile())
 {
  unless (($self->attracted()) && ($self->with_player()))
  {
   if (rand(100) < $self->mobile_chance())
   {
    my @dirs = ('east','west','south','north');
    my $dir = $dirs[rand(@dirs)];

    Utility->log(9,'character',$self->id() . " attempts to move $dir."); 

    $self->tell_user($dir);

    World->get_object($self->location())->go($self,$dir);
   }
  }
 }
 
}


sub with_player
{
 my $self = shift;

 foreach my $obj (World->get_environment($self))
 {
  if ($obj->type() eq 'Player')
  {
   return(1);
  }
 }

 return(0);
}


sub will_eat
{
 my $self = shift;
 my $objid = shift;

 push(@{$self->{attr}->{will_eat}},$objid);
}


sub hungry_for
{
 my $self = shift;
 my $objid = shift;

 foreach my $id (@{$self->{attr}->{will_eat}})
 {
  if ($id eq $objid)
  {
   return(1);
  }
 }

 return(0);
} 


sub add_verb_hook
{
 my $self = shift;
 my $verb = shift;
 my $code = shift;

 $self->{attr}->{verb_hooks}->{$verb} = $code;
}


sub has_verb_hook
{
 my $self = shift;
 my $verb = shift;

 if (defined $self->{attr}->{verb_hooks}->{$verb})
 {
  return(1);
 }
 return(0);
}


1;

