package Commands;

use strict;
no strict 'refs';
use vars qw($AUTOLOAD);
use Carp;

use Events;

use Data::Dumper;



sub parser 
{
 my $actor = shift;
 my $input = shift;

 my ($cmd,@param);

 my %global_aliases = (
        'n', 'go north',
        's', 'go south',
        'e', 'go east',
        'w', 'go west',
        'north', 'go north',
        'south', 'go south',
        'east', 'go east',
        'west', 'go west' );


 Utility->log(7,'commands',"actor: " . $actor->name() ." enters: $input");
 $input = Utility->filter_input($input);

 if ($input eq "") 
 { 
  $actor->prompt_user($actor->prompt());
  return;
 }

 foreach my $ga (keys %global_aliases)
 {
  
  if ($input eq $ga)
  {
   $input = $global_aliases{$ga};
  }
 }


 if (substr($input,0,1) eq '/')
 {
  # system com 
  my($cmd,@params) = split(/ /,$input);
  my $txt = join(' ',@params);

  my $cmd = lc($cmd);

  if ($cmd eq '/world')
  {
   Commands::world($actor);
  }
  elsif ($cmd eq '/say')
  {
   Commands::say($actor,$txt);
  }  
  elsif ($cmd eq '/shout')
  {
   Commands::shout($actor,$txt);
  }  
  elsif ($cmd eq '/who')
  {
   Commands::who($actor);
  }
  elsif ($cmd eq '/help')
  {
   Commands::help($actor,$txt);
  }
  elsif ($cmd eq '/dump')
  {
   Commands::dump($actor,$txt);
  }
  elsif ($cmd eq '/envobjs')
  {
   Commands::envobjs($actor);
  }
  elsif ($cmd eq '/teleport')
  {
   Commands::teleport($actor,$txt);
  }
  else
  {
   $actor->tell_user("Unknown command.");
  }

  $actor->prompt_user($actor->prompt());
  return;
 }

 else
 {
  
  # game com - parse this thing
  my (@words) = split(/ /,$input);
 

  my ($errtxt,$character,$verb,$objectsref,$leftoversref) = parse_cmd_objects($actor,@words);


  if ($errtxt ne '')
  {
   $actor->tell_user($errtxt);
   $actor->prompt_user($actor->prompt());
   return;
  }

  unless (defined $verb)
  {
   $actor->tell_user($actor->get_zone()->message('bad_command'));
   $actor->prompt_user($actor->prompt());
   return;
  }

  if (defined $character)
  {

   if ($character->has_verb_hook($verb->id()))
   {

    &{$character->get_verb_hook($verb->id())}($actor,$objectsref,$leftoversref);
   }
   else
   {
    my @responses = ('No, I\'m not your servant!', 'Do it yourself!', 'You must be crazy!', 'I don\'t want to!' );
    $actor->tell_user("The obstinate reply is, \"" . $responses[rand @responses] . "\"");
   }
  }
  else
  {
   $verb->command($actor,$objectsref,$leftoversref); 
  }

  $actor->commands_in_room($actor->commands_in_room() + 1);


  # World Events
  my (%world) = World->get_world_objects();
  foreach my $objid (keys %world)
  {
   $world{$objid}->tick($actor);
  } 

  # Local events

  foreach my $tickobj (World->get_objects_at($actor->location()))
  {

   if ($tickobj->sight())
   {
    $tickobj->sees($actor);
   }

   if (defined $tickobj->linger_action_ticks())
   {
    if ($actor->commands_in_room >= $tickobj->linger_action_ticks())
    {
     &{$tickobj->linger_action()}($tickobj,$actor);
     last;
    }
   }


   if (defined $tickobj->linger_action_chance())
   {
    my $chance = rand(100);
    if ($chance < $tickobj->linger_action_chance())
    {
     Utility->log(7,'commands',$actor->id() . " takes too many chances with " . $tickobj->id(). "($chance < " . $tickobj->linger_action_chance() . ')');
     &{$tickobj->linger_action()}($tickobj,$actor);
     last;
    }
   }


   if ($tickobj->has_saying())
   {
    Utility->log(7,'commands',$tickobj->id() . " has something to say to " . $actor->id());
    World->get_object($actor->location())->tell_room($tickobj->get_saying());
   }
  }
 }

 $actor->prompt_user($actor->prompt());
}




sub parse_cmd_objects
{
 my $actor = shift;
 my (@words) = @_;

 my $character;
 my $verb;
 my $errtxt;
 my @nouns;
 my @leftovers;

 my (%objectmap) = Utility->get_objectmap(World->get_environment($actor));   

 # add 'me'
 $objectmap{'me'} = $actor;

 # add 'here'
 $objectmap{'here'} = World->get_object($actor->location());

 # add 'walls'
 $objectmap{'walls'} = World->get_object($actor->location());

 

 for (my $pos = 0;$pos < @words;$pos++)
 {

  for (my $sz = (@words - 1 - $pos);$sz >= 0;$sz--)
  {
   my $cmpstr = join(' ', @words[$pos..($pos+$sz)]);

   if (defined $objectmap{$cmpstr})
   {
    # mark words used

    for (my $wipe = $pos;$wipe <= ($sz + $pos);$wipe++)
    {
     @words[$wipe] = '';
    }

    # check for ambigous

    if ($objectmap{$cmpstr} eq 'dup')
    {
     $errtxt = "Which $cmpstr?";
     return($errtxt);

    }

    # leading character name   

    my $objtype = $objectmap{$cmpstr}->type();

#print "parser: found " . $objectmap{$cmpstr}->id() . ": $objtype\n";

    if ($objtype eq 'Character')
    {

#print "found character: $cmpstr\n";

     if ($verb eq '')
     {

#print "setting character bc no verb yet\n";

      $character = $objectmap{$cmpstr};
     }
     else
     {
      push(@nouns,$objectmap{$cmpstr});
     }
    }
    elsif ($objtype eq 'Verb')
    {
     $verb = $objectmap{$cmpstr};
    }
    else
    {
     push(@nouns,$objectmap{$cmpstr});
    }
   }
  }
 }     

 foreach my $w (@words)
 {
  if ($w ne '')
  {
   push(@leftovers,$w);
  }
 }

 return($errtxt,$character,$verb,\@nouns,\@leftovers);
}




sub envobjs
{
 my $actor = shift;

 my (%nounmap) = Utility->get_objectmap(World->get_environment($actor));

 $actor->tell_user("\nEnvironment objects:\n");

 foreach my $noun (sort keys %nounmap)
 {
  if ($nounmap{$noun} eq 'dup')
  {
   $actor->tell_user(" $noun -> ambigous");
  }
  else
  {  
   $actor->tell_user(" $noun -> " . $nounmap{$noun}->id());
  }
 }

 $actor->tell_user(" ");
}








sub plugh
{
 my $actor = shift;

 if ($actor->mobile())
 {
  $actor->mobile(0);
  $actor->tell_user('You feel better.');
  World->get_object($actor->location())->tell_rest_of_room($actor,$actor->name() . " feels better.");
 }
 else
 {
  World->get_object($actor->location())->tell_rest_of_room($actor,$actor->name() . "'s face turns green... a moment later they vanish into thin air!");  
  
  $actor->set_location('electroshock13');
  $actor->tell_user(World->get_object('electroshock13')->description_from_inside($actor));  
  &{World->get_object('nurse')->linger_action()}(World->get_object('nurse'),$actor);
 }
}


sub quit
{
 my $actor = shift;

 $actor->return_all_inventory();

 World->get_object($actor->location())->tell_rest_of_room($actor,$actor->name() . " starts to convulse violently, and then disapears into thin air!"); 

 $actor->quit();
}




sub inventory
{
 my $actor = shift;
 
 $actor->tell_user("You are carrying the following:");

 foreach my $item (World->get_objects_at($actor->id()))
 {
  $actor->tell_user($item->name());
 }
 
 $actor->tell_user(" ");
}


sub take
{
 my $actor = shift;
 my $nounref = shift;
 my $leftoverref = shift;

 if (defined @{$nounref}[0])
 {
  if (defined @{$nounref}[1])
  {
   if (@{$nounref}[1]->location() eq $actor->id())
   { 
    @{$nounref}[0]->take_attempt($actor,@{$nounref}[1]);
   }
   else
   {
    $actor->tell_user("You are not carrying " . @{$nounref}[1]->name() . ".");
   }
  }
  else
  {
   @{$nounref}[0]->take_attempt($actor);
  }  
 }
 else
 {
  $actor->tell_user("Take what?");
 } 
}



sub kill
{
 my $actor = shift;
 my $nounref = shift;
 my $leftoverref = shift;

 if (defined @{$nounref}[0])
 {
  if (@{$nounref}[0]->type() eq 'Character')
  {
   my $rnd = rand(3);
   if ($rnd < 1)
   {
    $actor->tell_user("A wild punch and you miss.");
   }
   elsif ($rnd < 2)
   {
    $actor->tell_user("You beaned him!");
   }
   else
   {
    $actor->tell_user("A blow to the body!");
   }
  }
  else
  {
   $actor->tell_user("It does no good to attack " .  @{$nounref}[0]->name() . ".");
  }
 }
 else
 {
  $actor->tell_user("Attack what?");
 }
}


sub drop
{
 my $actor = shift;
 my $nounref = shift;
 my $leftoverref = shift;

 if (defined @{$nounref}[0])
 {
  if (@{$nounref}[0]->location() eq $actor->id())
  {
   @{$nounref}[0]->drop_attempt($actor); 
  }
  else 
  {
   $actor->tell_user("You are not carrying " . @{$nounref}[0]->name() . ".");    
  }
 }
 else
 { 
  $actor->tell_user("Drop what?");
 }
}


sub untie
{
 my $actor = shift;
 my $nounref = shift;
 my $leftoverref = shift;

 if (defined @{$nounref}[0])
 {
  if (@{$nounref}[0]->untieable())
  {
   @{$nounref}[0]->untie_attempt($actor);
  }
  else
  {
   $actor->tell_user("You can't do that.");
  }
 }
 else
 {
  $actor->tell_user("Untie what?");
 }
}



sub enter
{
 my $actor = shift;
 my $nounref = shift;
 my $leftoverref = shift;

 if (defined @{$nounref}[0])
 {
  if (@{$nounref}[0]->enterable() == 1)
  {
   Utility->log(7,'commands',$actor->name() . " enter attempt on " . @{$nounref}[0]->id());
   @{$nounref}[0]->enter_attempt($actor);
  }
  else
  {
   $actor->tell_user("You can't do that.");
  }
 }
 else
 {
  $actor->tell_user("Enter what?");
 }
}


sub exit
{
 my $actor = shift;
 my $nounref = shift;
 my $leftoverref = shift;

 if (defined @{$nounref}[0])
 {
  if (@{$nounref}[0]->id() eq $actor->location())
  {
   Utility->log(7,'commands',$actor->name() . " exit attempt on " . @{$nounref}[0]->id());
   @{$nounref}[0]->exit_attempt($actor);
  }
  else
  {
   $actor->tell_user("You are not in " . @{$nounref}[0]->name() . ".");
  }
 }
 else
 {
  $actor->tell_user("Exit what?");
 }
}



sub feed
{
 my $actor = shift;
 my $nounref = shift;
 my $leftoverref = shift;

 if (defined @{$nounref}[0])
 {
  if (defined @{$nounref}[1])
  {
   @{$nounref}[1]->feed_attempt($actor,@{$nounref}[0]);
  }
  else
  {
   $actor->tell_user("Feed " . @{$nounref}[0]->name() . " with what?");
  }
 }
 else
 {
  $actor->tell_user("Feed what?");
 }
}

sub give
{
 my $actor = shift;
 my $nounref = shift;
 my $leftoverref = shift;

 if (defined @{$nounref}[0])
 {
  if (defined @{$nounref}[1]) 
  {
   @{$nounref}[0]->give_attempt($actor,@{$nounref}[1]);
  }
  else
  {
   $actor->tell_user("Give " . @{$nounref}[0]->name() . " to what?");
  }
 }
 else
 {
  $actor->tell_user("Give what?");
 }
}



sub unlock
{
 my $actor = shift;
 my $nounref = shift;
 my $leftoverref = shift;

 if (defined @{$nounref}[0])
 {
  if (defined @{$nounref}[1])
  {
   if (@{$nounref}[1]->location() eq $actor->id())
   {
    @{$nounref}[0]->unlock_attempt($actor,@{$nounref}[1]);
   }
   else
   {
    $actor->tell_user("You are not carrying " . @{$nounref}[1]->name() . ".");
   }
  }
  else
  {
   $actor->tell_user("Unlock ". @{$nounref}[0]->name() . " with what?");
  }
 }
 else
 {
  $actor->tell_user("Unlock what?");
 } 
}



sub put
{
 my $actor = shift;
 my $nounref = shift;
 my $leftoverref = shift;

 if (defined @{$nounref}[0])
 {
  if (defined @{$nounref}[1])
  {
   if (@{$nounref}[0]->location() eq $actor->id())
   {
    @{$nounref}[0]->put_attempt($actor,@{$nounref}[1]);
   }
   else
   {
    $actor->tell_user("You are not carrying " . @{$nounref}[0]->name() . ".");
   }
  }
  else
  {
   $actor->tell_user("Put ". @{$nounref}[0]->name() . " where?");
  }
 }
 else
 {
  $actor->tell_user("Put what?");
 }
}







sub eat
{
 my $actor = shift;
 my $nounref = shift;
 my $leftoverref = shift;

 if (defined @{$nounref}[0])
 {
  if (@{$nounref}[0]->location() eq $actor->id())
  {
   @{$nounref}[0]->eat_attempt($actor);
  }
  else
  {
   $actor->tell_user("You are not carrying " . @{$nounref}[0]->name() . ".");
  }
 }
 else
 {
  $actor->tell_user("Eat what?");
 }
}


sub help
{
 my $self = shift;
 my $input = shift;

 if ($input ne '')
 {
  $input =~ s/\///g;

  if (-e "text/help/$input.txt")
  {
   Utility->display_file($self,"text/help/$input.txt");
  }
  else
  {
   $self->tell_user("No help for $input");
  } 
 }
 else
 {
  Utility->display_file($self,'text/help/general.txt');
 }
}


sub open
{
 my $actor = shift;
 my $nounref = shift;
 my $leftoverref = shift;


 if (defined @{$nounref}[0])
 {
  @{$nounref}[0]->open_attempt($actor);
 }
 else
 {
  $actor->tell_user("Open what?");
 }
}


sub close
{
 my $actor = shift;
 my $nounref = shift;
 my $leftoverref = shift;

 if (defined @{$nounref}[0])
 {
  @{$nounref}[0]->close_attempt($actor);
 }
 else
 {
  $actor->tell_user("Close what?");
 }
}




sub who
{
 my $self = shift;     
 my (@param) = $_;

 $self->tell_user("\nCurrent Users:\n");

 my %sessions = %{Utility->reference("sessions")};

 foreach my $h_user (sort keys %sessions) 
 {
  my $user = $sessions{$h_user};
  my $idle = time - $user->idlesince();
  $self->tell_user(sprintf(" %-14s %-14s %-14s %-14s",ucfirst($user->name()),$user->status(),$idle,World->get_object($user->location())->name()));
 }

 $self->tell_user(" ");

}


sub world
{
 my $self = shift;
 my (@param) = $_;

 $self->tell_user("\nThe World:\n");

 my (%world) = World->get_world_objects();

 $self->tell_user(sprintf(" %-16s %-10s %-16s %-4s %-10s",'[ id ]','[ type ]','[ location ]','[s#]','[ status ]'));

 foreach my $objid (sort keys %world)
 {
  $self->tell_user(sprintf(" %-16s %-10s %-16s %-4s %-10s",$world{$objid}->id(),$world{$objid}->type(),$world{$objid}->location(),$world{$objid}->serial(),$world{$objid}->status()));
 }

 $self->tell_user(" ");

}





sub say {
        my $self = shift;
        my $what =  join(' ', @_);

        my ($action);
        $what =~ s/\s+$//;
        my $punct = substr($what,-1,1);
        if ($punct eq "!") { $action = "exclaim" }
        elsif ($punct eq "?") { $action = "ask" }
        else { $action = "say" }

        $self->tell_user("You ".$action.", \"".$what."\"");
        World->get_object($self->location())->tell_rest_of_room($self,$self->name() ." ".$action."s, \"$what\"");        
}

sub shout {
        my $self = shift;
        my $what =  join(' ', @_);

        $self->tell_user("You shout, \"".$what."\"");
        World->broadcast($self->name() . " shouts, \"$what\"",$self);                     
}



sub dump
{
 my $actor = shift;
 my $txt = shift;

 my (@words) = split(/ /,$txt);

 if (World->valid_object($words[0]))
 {
  my $dobj = World->get_object($words[0]);
  $actor->tell_user("\nDump: " . $dobj->name() . "\n");
  my $d = Dumper($dobj);
  $actor->tell_user($d);
  $actor->tell_user(" ");
 }
 else
 {
  $actor->tell_user("Object not found.");
 }
}


sub teleport
{
 my $actor = shift;
 my $txt = shift;

 my (@words) = split(/ /,$txt);

 if (World->valid_object($words[0]))
 {
  $actor->set_location(World->get_object($words[0])->id());
 }
 else
 {
  $actor->tell_user('Unknown destination.');
 }
}



sub look
{
 my $actor = shift;
 my $nounref = shift;
 my $leftoverref = shift;

 my $target = @{$nounref}[0];

 if (defined $target)
 {
  # lets you see things inside other things, but its authentic to original
  $actor->tell_user($target->look_at());
 }
 else
 {
  my $lookobj = World->get_object($actor->location());
  $actor->tell_user($lookobj->description_from_inside($actor));
 }
}



sub exits
{
 my $self = shift;

 my $locobj = World->get_object($self->location());

 $self->tell_user("Obvious Exits:");

 $self->tell_user($locobj->obvious_exits());

}


sub go
{
 my $actor = shift;
 my $nounref = shift;
 my $leftoverref = shift;

 World->get_object($actor->location())->go($actor,@{$leftoverref}[0]);

}

1;  # so the require or use succeeds

