package MUD;


use strict;				# keeps us honest - and good coders
no strict 'refs';			# allows us to get away with a *few* things!

use GameConfig;
use Net::hostent; 			# used to find the client address from the IP's
use Socket;				# necessary for the SocketFactory
use Compress::Zlib;			# compression for MCCP
use POE qw(
		Wheel::SocketFactory
		Wheel::ReadWrite 
		Driver::SysRW 
		Filter::Line 
		Filter::Stream);	# all the necessary POE pieces

use Utility;
use World;
use Messages;

my (%addresses);		# Initialize and localize vars.
my $world;

my $config = GameConfig->new(); 


sub new {

# create world... ?

# World->init_world();

#create a new the main server session

	POE::Session->create(
		inline_states =>
			{	_start			=> \&server_start, 	# server _start handler
				_stop			=> \&server_stop,   	# server _stop handler
				event_success		=> \&server_accept, 	# server connection handler
				event_failure		=> \&server_error,  	# server error handler
				tictoc			=> \&server_beat,	# server timer event handler
			}
	);
	
	#create a new heartbeat timer
	POE::Session->create(
		inline_states =>
			{	_start			=> \&timer_start, 	# timer _start handler
				_stop			=> \&timer_stop,   	# timer _stop handler
				set_alarm		=> \&timer_set, 	# server connection handler
				signal_check		=> \&timer_sig,  	# server error handler
			}
	);
	
 	$poe_kernel->run();

	return $poe_kernel;
}

##########################################################################


sub server_start {
	my ($heap,$kernel) = @_[HEAP,KERNEL];
	my $port = $config->{'PORT'};

	Utility->kernel($kernel);

	$heap->{listener} = POE::Wheel::SocketFactory->new(
		BindPort       => $port,
		Reuse          => 'yes',           	# reuse the port right away
		SuccessEvent   => 'event_success', 	# event to send on connection
		FailureEvent   => 'event_failure'  	# event to send on error
	);

	$kernel->sig('TIC', 'tictoc');			# registers the event handler

	Utility->log(0,'server',"listening on port $port");
#??	World->world_init();
}

 
sub server_accept {
	my ($accepted_socket, $peer_address, $peer_port) = @_[ARG0, ARG1, ARG2];
	$peer_address = inet_ntoa($peer_address);
	Utility->log(1,'server',"connection from $peer_address : $peer_port");
  
	POE::Session->create( inline_states => {
						_start      	=> \&session_start, # _start event handler
						_stop       	=> \&session_stop,  # _stop event handler
						line_input 	=> \&session_input, # input event handler
						io_error   	=> \&session_error, # error event handler
						out_flushed 	=> \&session_flush, # flush event handler
						hear        	=> \&session_heard, # someone said something
						prompt		=> \&session_prompt, # prompt user for input
						tatump		=> \&session_beat },	# user event handler (hearbeat)
						args => [ $accepted_socket, $peer_address, $peer_port ]
					);
}


sub server_error {
  my ($heap, $operation, $errnum, $errstr) = @_[HEAP, ARG0, ARG1, ARG2];
  Utility->log(0,'server',"ERROR $operation error $errnum: $errstr");
  delete $heap->{listener};
}


sub server_stop {
  Utility->log(0,'server',"stopped");
  delete $_[HEAP]->{listener};
}


sub server_beat {
	# do worldly stuff like change the weather
	World->world_beat(); 

	Utility->log(9,'server',"heartbeat");

#	my $size = Utility->size("mud_to_do");				# get to_do's size
#	for (my $i = 0; $i < $size; $i++) {
#		my $command = Utility->get("mud_to_do");		# shift off the first command
#		my $response = eval $command;				# evaluate the command (run it)
#		if ($@) { 									# respond to errors
#			Utility->error_log("Error running command: $@");
#		}
#		if ($response == 1) {
#			Utility->add("mud_to_do",$command);		# if it returns 1, push back on to_do
#		} elsif ($response == 3.1415) {
#			die;
#		} elsif ($response == 999) {
#			my $program_source = $settings{'perl'}.
#				" ".$settings{'driverdir'}."mud.pl";
#			exec $program_source;
#			die;
#		}
#	}
}


sub timer_start {
	my $kernel = $_[KERNEL];
	$kernel->delay('set_alarm', $config->{'SERVER_BEAT'});
}


sub timer_set {
	my $kernel = $_[KERNEL];
	$kernel->signal($kernel, 'TIC');	# signal the servers
	$kernel->signal($kernel, 'TOCK');	# signal the sessions
	$kernel->delay('set_alarm', $config->{'SERVER_BEAT'});
}


sub timer_stop {
	Utility->log(1,'server',"timer stop");
}

##########################################################################


sub session_start 
{
 my ($kernel,$heap, $session, $accepted_socket, $address) = @_[KERNEL, HEAP, SESSION, ARG0, ARG1];

 Utility->log(1,'session', "new session with $address");

 if (my $h = gethostbyaddr($address)) { $addresses{$session} = $h }
	else { $addresses{$session} = $address }

 $heap->{readwrite} = new POE::Wheel::ReadWrite
 (	Handle       => $accepted_socket,       	# read/write on this handle
	Driver       => new POE::Driver::SysRW, 	# using sysread and syswrite
	InputFilter  => new POE::Filter::Line,  	# filtering I/O as lines
	OutputFilter => new POE::Filter::Stream,  	# filtering I/O as stream (allows prompt)
	InputEvent   => 'line_input',     		# generate line_input on input
	ErrorEvent   => 'io_error',       		# generate io_error on error
	FlushedEvent => 'out_flushed',    		# generate out_flushed on flush
 );	

 $heap->{address}=$address;
 $kernel->sig('TOCK', 'tatump');						# registers the user event handler

 $kernel->call($session, "hear", $config->{'BANNER'}); 	# login rules
 $kernel->call($session, "prompt", $config->{'NAMEPROMPT'});				# login prompt	
}


sub session_stop {
	my ($session,$heap) = @_[SESSION,HEAP];
	Utility->log(1,'session',"STOP session: $session heap: $heap");

#	if (my $self = Utility->find_session($session)) {
	#	Utility->inform($self->name()." has left the game.");
#		Utility->delete_session($session);
#		delete $addresses{$session};
#
#	}
	delete $heap->{readwrite};
#        die;
}


sub session_input {
	my ($kernel, $session, $heap, $input) = @_[KERNEL, SESSION, HEAP, ARG0];
	Utility->log(8,'session',"SESSION_INPUT->session: $session input: $input");

	1 while ($input =~ s/[^\x08\x7F][\x08\x7F]//g);
	$input =~ tr[\x08\x7F][]d;
	

	# Parse the client's input for commands, and handle them.
	if (my $self = Utility->find_session($session)) {
			$self->idlesince(time);
			&{$self->go_to()}($self,$input);		# send the user input to whereever
		
	}
        else {
		Utility->log(2,'session',"sending $input to login");
		my $po = Player->login($kernel, $heap, $session, $input); 
	}
}


sub session_error {
	my ($kernel, $session, $operation, $errnum, $errstr) =
		@_[KERNEL, SESSION, ARG0, ARG1, ARG2];

        if (my $self = Utility->find_session($session)) 
        {
         Utility->del("sessions",$session);                                      # currently deletes session
         delete $addresses{$session};
         Utility->inform($self->name().' has disconnected.');
        }
        else
        {
	 Utility->inform("[$operation error $errnum: $errstr]");
        }
	Utility->log(1,'session',"SESSION_ERROR->session: $session got $operation error $errnum: $errstr" );
	delete $_[HEAP]->{readwrite};
}


sub session_flush {
		my $self = shift;
}


sub session_heard {
	my ($heap, $session, $what) = @_[HEAP, SESSION, ARG0];
	Utility->log(8,'session',"SESSION_HEARD->session: $session said $what");
	my $self = Utility->find_session($session);
	if($what) {
		if($self) 
                { 
                 eval { $heap->{readwrite}->put($what."\x0D\x0A"); };
                 if ($@)
                 {
                  Utility->log(1,'session',"session_heard error: $@");
                 }
		} 
                else 
                { 
			# to stop: Can't call method "put" on an undefined value
			# did not work
                 eval { $heap->{readwrite}->put($what."\x0D\x0A"); };
                 if ($@) 
                 {
                  Utility->log(1,'session',"session_heard error: $@");
                 }
		}
	}
}


sub session_prompt {
	my ($heap, $session, $what) = @_[HEAP, SESSION, ARG0];
	eval { $heap->{readwrite}->put($what); };
        if ($@)
        {
         Utility->log(1,'session',"session_heard error: $@");
        }
}


sub session_beat {
	my ($kernel, $session) = @_[KERNEL, SESSION];
	my $self = Utility->find_session($session);		# verify user is still logged in
	if ($self)
 	{
		# put here to stop it crashing on login
		if ($self->status() eq "active") {		# verify user is active
			$self->do_todo();			# process users heartbeat events
			
		}
	}

	Utility->log(9,'session',"heartbeat for session $session");
	
	$kernel->sig_handled();		
}

1;
