package Object;

use strict;
use Clone::Fast qw( clone );

use vars qw($AUTOLOAD);
no strict 'refs';


my $serialnum = 0;


sub new  
{
 my $class = shift;
 my $newid = shift;


 if (defined $newid)
 {
  my $self = {};

  bless $self, $class;  

  World->register_object($newid,$self);

  $self->{serial} = $serialnum;
  $serialnum++;

  $self->{id} = $newid;
  $self->{type} = $class;
  $self->{attr} = ();
  $self->{states} = ();
  $self->{location} = 'void';

  $self->visible(1);

  Utility->log(5,'object',"new $class: id $newid, serial #" . $self->serial());


  return $self;
 }
 else
 {
  Utility->log(1,'object',"failed to create new verb because no id was given");
 }



}

sub id
{
 my $self = shift;

 return($self->{id});
}

sub type
{
 my $self = shift;

 return($self->{type});
}

sub serial
{
 my $self = shift;

 return($self->{serial});
}


sub location
{
 my $self = shift;
 my $loc = shift;

 if (defined $loc)
 {
  $self->{location} = $loc;
 }

 return($self->{location});
}



sub name
{
 my $self = shift;
 my $txt = shift;

 if (defined $txt)
 {
  $self->{attr}->{name} = $txt;
 }

 if ($self->{attr}->{name} ne '')
 {
  return($self->{attr}->{name});
 }

 return(ucfirst($self->id()));
}



sub current_state
{
 my $self = shift;
 my $state = shift;

 if (defined $state)
 {
  $self->{current_state} = $state;
 }

 return($self->{current_state});
}


sub default_state
{
 my $self = shift;
 my $state = shift;

 if (defined $state)
 {
  $self->{default_state} = $state;
 }

 return($self->{default_state});
}



sub save_state
{
 my $self = shift;
 my $state = shift;

 $self->{states}->{$state} = clone( $self->{attr} );
 
}

sub load_state
{
 my $self = shift;
 my $state = shift;

 $self->{attr} = clone ( $self->{states}->{$state} );
 $self->current_state($state);

}



sub AUTOLOAD {
        my $self = shift;
        my $name = $AUTOLOAD;

        $name =~ s/.*:://;

        return unless $name=~/[^A-Z]/;  # skip DESTROY and all-cap methods

        $self->{attr}->{lc $name} = shift if @_;

        Utility->log(9,'noun',"autoload: " . $self->id() . ", $name = " . $self->{attr}->{lc $name});

        return $self->{attr}->{lc $name};
}




sub tick
{
 my $self = shift;
 my $actor = shift;

 if (defined $self->{attr}->{tick_event})
 {
  &{$self->{attr}->{tick_event}}($self);
 }

 Utility->log(9,'noun',$self->id . ' gets tick for ' . $actor->name());
}




sub get_aliases
{
 my $self = shift;

 return(@{$self->{attr}->{aliases}});
}


sub add_alias
{
 my $self = shift;
 my $alias = shift;

 push(@{$self->{attr}->{aliases}},$alias);
}

sub message
{
 my $self = shift;
 my $msgname = shift;

 if (defined $self->{attr}->{messages}->{$msgname})
 {
  return($self->{attr}->{messages}->{$msgname});
 }
 
 return($msgname);
}


sub add_message
{
 my $self = shift;
 my $msg = shift;
 my $txt = shift;

 $self->{attr}->{messages}->{$msg} = $txt;
}


sub is_message
{
 my $self = shift;
 my $msgname = shift;

 if (defined $self->{attr}->{messages}->{$msgname})
 {
  return(1);
 }
 
 return(0);
}


sub description
{
 my $self = shift;
 my $txt = shift;

 if (defined $txt)
 {
  $self->{attr}->{description} = $txt;
 }

 if ($self->{attr}->{description} ne '')
 {
  return($self->{attr}->{description});
 }

 return(ucfirst($self->name()) . " is here.");
}


sub get_all_attributes
{
 my $self = shift;

 return($self->{attr});
}



sub is_phantom_at
{
 return(0);
}


sub tell_room
{
 my $self = shift;
 my $txt = shift;

 foreach my $obj ($self->get_listeners())
 {
  $obj->tell_user($txt);
 }

}

sub tell_rest_of_room
{
 my $self = shift;
 my $source = shift;
 my $txt = shift;

 foreach my $obj ($self->get_listeners())
 {
  if ($obj->id() ne $source->id())
  {
   $obj->tell_user($txt);
  }
 }

}


sub get_listeners
{
 my $self = shift;

 my @listeners;

 foreach my $obj (World->get_objects_at($self->id()))
 {
  if ($obj->type() eq 'Player') # include mobs?
  {
   push(@listeners,$obj);
  }
 }

 return(@listeners);
}





sub DESTROY
{
 my $self = shift;

 World->destroy_object($self);
}


1;

