package Room;

use strict;
no strict 'refs';
use base 'Noun';

use Data::Dumper;

my %exits;


sub new  {

        my $class = shift;
        my $id = shift;

        my $self = { %exits };

        my $self = $class->SUPER::new($id);

        bless ($self, $class);   

        Messages->set_room_default_messages($self);

        return $self;
}



sub add_exit
{
 my $self = shift;

 my $dir = shift;
 my $dest = shift;
 my $doorid = shift;

 $self->{attr}->{exits}->{$dir}->{dest} = $dest;
 if (defined $doorid)
 {
  $self->{attr}->{exits}->{$dir}->{doorid} = $doorid;
 }
}


sub obvious_exits
{
 my $self = shift;

 my $txt;
 
 foreach my $dir (keys %{$self->{attr}->{exits}})
 {
    if ($txt) { $txt .= "\n"; }

    my $doorid = $self->{attr}->{exits}->{$dir}->{doorid};

    if (defined $doorid)
    {

     if (World->valid_object($doorid))
     {
      $txt .= $dir;
      my $door = World->get_object($doorid);

      if ($door->closed()) { $txt .= " (door closed)"; }
     } 
    }
    else
    { 
     $txt .= $dir;
    }
 } 

 return($txt);
}


sub is_exit
{
 my $self = shift;
 my $exit = shift;

 if (defined $self->{attr}->{exits}->{$exit})
 {
  return(1);
 }
 
 return(0);
}


sub go
{
 my $self = shift;
 my $obj = shift;
 my $dir = shift;

 if ($self->is_exit($dir))
 {
  my $dest = $self->{attr}->{exits}->{$dir}->{dest};
  my $doorid = $self->{attr}->{exits}->{$dir}->{doorid};

  if (defined $doorid)
  {
   my $door = World->get_object($self->{attr}->{exits}->{$dir}->{doorid});

   if ($door->locked())
   {
    if ($door->is_message('door_locked'))
    {
     $obj->tell_user($door->message('door_locked'));
    }
    else
    {
     $obj->tell_user('The door is locked.');
    }
   }
   elsif ($door->closed())
   {
    if ($door->is_message('door_closed'))
    {
     $obj->tell_user($door->message('door_closed'));
    }
    else
    {
     $obj->tell_user('The door is closed.');
    }
   }
   elsif (($obj->type() eq 'Character') && (World->get_object($dest)->no_characters()))
   {
    Utility->log(7,'room',$obj->name() . ' tries to move into a no character room.');
   }
   else
   {
    # move user
    $obj->set_location($dest);
    # announce departure

    $self->annouce_departure($obj,$dir,$door);

    # annouce entry

    World->get_object($dest)->annouce_arrival($obj,$dir,$door);

    $obj->tell_user(World->get_object($dest)->description_from_inside($obj));
   }
  }
  else
  {
   # move user
   $obj->set_location($dest);
   # announce departure

   $self->annouce_departure($obj,$dir);

   # annouce entry

   World->get_object($dest)->annouce_arrival($obj,$dir);

   unless (World->get_object($dest)->{attr}->{ "suppress_look_on_entry_from_$dir" } == 1)
   {
    $obj->tell_user(World->get_object($dest)->description_from_inside($obj));
   }
  }
 }
 else
 {
  $obj->tell_user($self->message('no_such_exit'));
 }
}



sub annouce_departure
{
 my $self = shift;
 my $obj = shift;
 my $dir = shift;
 my $door = shift;


 my $otxt = '';
 my $txt = '';

 if (defined $door)
 {
  # handle door 
  if ($door->is_message("door_odeparture_$dir"))
  {
   $otxt = $door->message("door_odeparture_$dir");
  }
  elsif ($door->is_message("door_odeparture"))
  {
   $otxt = $door->message("door_odeparture");  
  }

  if ($door->is_message("door_departure_$dir"))
  {
   $txt = $door->message("door_departure_$dir");
  }
  elsif ($door->is_message("door_departure"))
  {
   $txt = $door->message("door_departure");
  }
  
 }

 if ($otxt eq '')
 {
  if ($self->is_message("room_odeparture_$dir"))
  {
   $otxt = $self->message("room_odeparture_$dir"); 
  }
  elsif ($self->is_message('room_odeparture'))
  {
   $otxt = $self->message('room_odeparture');
  }
  elsif ($obj->is_message('object_odeparture'))
  {
   $otxt = $obj->message('object_odeparture');
  }
  else
  {
   $otxt = "goes $dir.";
  }
 }

 if ($txt eq '')
 {
  if ($self->is_message("room_departure_$dir"))
  {
   $txt = $self->message("room_departure_$dir");
  }
  elsif ($self->is_message('room_departure'))
  {
   $txt = $self->message('room_departure');
  }
  elsif ($obj->is_message('object_departure'))
  {
   $txt = $obj->message('object_departure');
  }
 }

 $self->tell_room(ucfirst($obj->name()) . " " . $otxt);
 if ($txt ne '') { $obj->tell_user($txt); }
}



sub annouce_arrival
{
 my $self = shift;
 my $obj = shift;
 my $dir = shift;
 my $door = shift;

 my $otxt = '';
 my $txt = '';

 if (defined $door)
 {
  # handle door
  if ($door->is_message("door_oarrival_$dir"))
  {
   $otxt = $door->message("door_oarrival_$dir");
  }
  elsif ($door->is_message("door_oarrival"))
  {
   $otxt = $door->message("door_oarrival");
  }

  if ($door->is_message("door_arrival_$dir"))
  {
   $txt = $door->message("door_arrival_$dir");
  }
  elsif ($door->is_message("door_arrival"))
  {
   $txt = $door->message("door_arrival");
  }
 }

 if ($otxt eq '')
 {
  if ($self->is_message("room_oarrival_$dir"))
  {
   $otxt = $self->message("room_oarrival_$dir");
  }
  elsif ($self->is_message('room_oarrival'))
  {
   $otxt = $self->message('room_oarrival');
  }
  elsif ($obj->is_message('object_oarrival'))
  {
   $otxt = $obj->message('object_oarrival');
  }
  else
  {
   $otxt = "enters from the " . Utility->convert_direction($dir) . ".";
  }


  if ($self->is_message("room_arrival_$dir"))
  {
   $txt = $self->message("room_arrival_$dir");
  }
  elsif ($self->is_message('room_arrival'))
  {
   $txt = $self->message('room_arrival');
  }
  elsif ($obj->is_message('object_arrival'))
  {
   $txt = $obj->message('object_arrival');
  }
 }

 $self->tell_rest_of_room($obj,ucfirst($obj->name()) . " " . $otxt);
 if ($txt ne '') { $obj->tell_user($txt); }
}




sub get_doors_dis
{
 my $self = shift;

 my @doors;

 foreach my $exit (keys %{$self->{attr}->{exits}})
 {
  if (defined $self->{attr}->{exits}->{$exit}->{doorid})
  {
   push(@doors,World->get_object($self->{attr}->{exits}->{$exit}->{doorid}));
  }
 }

 return(@doors);
}




1;

