package Utility;

# INCLUDED LIBRARY FILES
use strict;
#no strict 'refs';

use Sys::Syslog qw(:DEFAULT setlogsock);

use Data::Dumper;

#########################################################################
# CONSTANTS AND GLOBAL VARIABLES

my (%connected_sessions,$kernel);
my $session_count = 0;

my $scheduleid	= 0;			# the schedule id increments so each entry has a unique id

my $debuglevel = 7;

# here is where the master lists of the mud reside
# maybe it should be moved to World.pm

my $lists = {
	sessions	=>	{}, 	# sessions list
	schedule	=>	{}, 	# schedule entries
	monsters	=>	{}, 	# npc mobiles list
	talkers		=>	{}, 	# talker objects list
	place		=>	{}, 	# rooms loaded
	mud_to_do	=>	[], 	# to do list to be run before the next heartbeat
	cps		=>	{}, 	# commands per second
	rooms_leave	=>	{}, 	# rooms that have 'leaves' messages pending
	rooms_arrive	=>	{}, 	# rooms that have 'arrives' messages pending
};

#########################################################################
# FUNCTION DEFINITIONS

sub new  { }

sub DESTROY { }

#########################################################################
# FILTER_INPUT - given a string, filter out the badness and return it.

# needs testing and refining, this only works for certain clients on certain platforms
sub filter_input {
	my $class = shift;
	my $input = shift;

	# character that was stopping kmuddy
	my $c31 = chr(31);
	
	$_ = $input;
	#s/$c31//g;	
	# Don't let the user embed commands. Could do nasty, nasty things.
	# these babies stopped kmuddy from connectio
# 	s/\x01/\./g;	#todo - add more intensive filtering later
# 	s/\x02/\./g;	#maybe trigger a message to an admin when detected
	s/\x01//g;	#todo - add more intensive filtering later
	s/\x02//g;	#maybe trigger a message to an admin when detected
 	s/^$c31//g;	#kmuddy seems to send this, remove it and it works. uhuh, 
	# Clean up whitespace. 
# 	s/\s/ /g;
 	s/^\s//g;
	s/\s+$//g;
	$input = $_;

	return $input;
}

#########################################################################
# WRAP - used for text wrap on the terminal screens

sub wrap
{
	my $class = shift;
	my ($columns, @text) = @_;

	my $break = '\s';
	my $r = "";
	my $t = join(" ",@text);
	my $ll = $columns - 1;
	my $nl = "";
	my $remainder = "";

	if ($columns == 0 || $columns eq "") { return $t };

	while ($t !~ /^\s*$/) {
		if ($t =~ s/^([^\n]{0,$ll})($break|\Z(?!\n))//xm) {
			$r .= $nl . $1;
			$remainder = $2;
		} elsif ($t =~ s/^([^\n]{$ll})//) {
			$r .= $nl . $1;
			$remainder = "\n";
		} else {
			Utility->log(1,'utility',"An error occurred during textwrap.");
		}
			
		$nl = "\r\n";
	}
	$r .= $remainder;
	$r .= $t if $t ne "";
	return $r;
}

#########################################################################
# inform - broadcasts an announcement of the passed string to everyone on the mud

sub inform {
	my $class = shift;
	my $kernel = Utility->kernel();
	my $what = join('', @_);
	my %sessions = %{Utility->reference("sessions")};
	foreach my $session (keys (%sessions)) {
		$kernel->call($session, "hear", "$what");
	}
}

#########################################################################
# Session functions - maintains list of active sessions

# returns a user object given a session id
sub find_monster_session {
	my $class = shift;
	my $session = shift;
	my %sessions = %{Utility->reference("monsters")};
	if (exists $sessions{$session}) { return $sessions{$session} }
	else { return 0; }
}

# returns a user object given a session id
sub find_session {
	my $class = shift;
	my $session = shift;
	my %sessions = %{Utility->reference("sessions")};
	if (exists $sessions{$session}) { return $sessions{$session} }
	else { return 0; }
}

# returns a session id given a user object
sub find_user {
	my $class = shift;
	my $user = shift;
	my %sessions = %{Utility->reference("sessions")};
	foreach my $session (keys %sessions) {
		if ($user eq $sessions{$session}) { return $session	}
	}
	return 0;
}	

# returns a session id given a username
sub find_username {
	my $class = shift;
	my $user_name = shift;
	my %sessions = %{Utility->reference("sessions")};
	foreach my $user (values %sessions) {
		if (lc($user_name) eq $user->name()) { return $user	}
	}
	return 0;
}	

#########################################################################
# add, find, return info about objects hashes or arrays

## adds a value to either a hash or an array
sub add {
	my $class = shift;
	my ($refval, $key, $value) = @_;
	if (ref($lists->{$refval}) eq "ARRAY") {			# check if ARRAY
		if ($value && $value < $lists->size($refval)) {		# check existance and size of value
			splice(@{$lists->{$refval}}, $value, 0, $key);	# key in value array position
		} else {
			push @{$lists->{$refval}}, $key;		# push $key onto end of ARRAY
		}
		return 1;
	} elsif (ref($lists->{$refval}) eq "HASH") {			# check if HASH
		$lists->{$refval}->{$key} = $value;			# create HASH key{value}
		return 1;
	}
	return 0;							# if neither, ditch
}

## deletes a value from either a hash or an array
sub del {
	my $class = shift;
	my ($refval, $key, $value) = @_;
	if (ref($lists->{$refval}) eq "ARRAY") {			# check if ARRAY
		if ($value && $value < $lists->size($refval)) {# check existance and size of value
			splice(@{$lists->{$refval}}, $value, 1);	# key in value array position
		} else {
			pop @{$lists->{$refval}};			# pop $key off end of ARRAY
		}
		return 1;
	} elsif (ref($lists->{$refval}) eq "HASH") {			# check if HASH
		delete $lists->{$refval}->{$key};			# create HASH key{value}
		return 1;
	}
	return 0;							# if neither, ditch
}

## returns a value of either a hash or an array
sub get {
	my $class = shift;
	my ($refval, $key) = @_;
	if (ref($lists->{$refval}) eq "ARRAY") {			# check if ARRAY
		if ($key) {						# check for key
			return $lists->{$refval}->[$key];		# return value at key array position
		} else {									# or
			return shift @{$lists->{$refval}};		# shift off first array value
		}
	} elsif (ref($lists->{$refval}) eq "HASH") {			# check if HASH
		return $lists->{$refval}->{$key};			# return value of HASH{key}
	}
	return 0;							# if neither, ditch
}

## returns size of either a hash or an array
sub size {
	my $class = shift;
	my ($refval) = @_;
	if (ref($lists->{$refval}) eq "ARRAY") {			# check if ARRAY
		my $size =  scalar(@{$lists->{$refval}});		# $size = array length +1
		if ($size < 0) {return 0;}				# correct any oddities
		else {return $size;}					# return size
	} elsif (ref($lists->{$refval}) eq "HASH") {			# check if HASH
		my $size = scalar(keys %{$lists->{$refval}});		# $size = length of keys + 1
		if ($size < 1) {return 0;}				# correct oddities
		else {return $size;}					# return size
	}
	return 0;							# if neither, ditch
}

## returns the ref of either a hash or an array
sub reference {
	my $class = shift;
	my ($refval) = @_;
	if (ref($lists->{$refval}) eq "ARRAY") {			# check if ARRAY
		return $lists->{$refval};				# return the reference
	} elsif (ref($lists->{$refval}) eq "HASH") {			# check if HASH
		return $lists->{$refval};				# return the reference
	}
	return 0;							# if neither, ditch
}

sub check {
	my $class = shift;
	my ($refval,$key) = @_;
	if (ref($lists->{$refval}) eq "ARRAY") {			# check if ARRAY
		foreach (@{$lists->{$refval}}) {
			if ($key eq $_) {return 1;}
		}
	} elsif (ref($lists->{$refval}) eq "HASH") {			# check if HASH
		if (exists $lists->{$refval}->{$key}) {
			return $lists->{$refval}->{$key};
		}
	}
	return 0;							# if neither, ditch
}

#########################################################################
# misc functions

# silly little sub that returns a boolean response if an array contains a value
sub isin {  # Return if $var isin @
	my $class = shift;
	my ($var, @set) = @_;
	foreach (@set) { if ($var eq $_) {return 1;} }
	return 0;
}

# returns a room object given a room_name and loads the file if necessary
sub find_place {
	my $class = shift;
	my $place_name = shift;
	my ($here);
	if (exists $lists->{"place"}->{$place_name}) {
		return $lists->{"place"}->{$place_name}; 
	} 
 return(0);
}	

sub ret_rooms {
	my $class = shift;
# 	foreach ($lists->{"place"}) {
# 		print "ROOM:$_\n";
# 	}
	my %rooms = %{Utility->reference("place")};
	my @rooms = keys %rooms;
	return (@rooms);
}

sub convert_direction {
	my $class = shift;
	my $exit = shift;
	my ($direction);	
	my %convert = 	(
		'north'		=>	"south",
		'south'		=>	"north",
		'east'		=>	"west",
		'west'		=>	"east",
		'up'		=>	"down",
		'down'		=>	"up",
		'leave'		=>	"arrive",
			);
	if (exists $convert{$exit}) {
		$direction = $convert{$exit};
	} else { $direction = $exit; }
	return $direction;
}

sub convert_multiple {
	my $class = shift;
	my (@array) = @_;
	my (@unique, @duplicates, @list, @once) = ();
	my (%count1, %count2, %count3, %count4, $num);
	
	# count unique elements in array
	@unique = grep { ++$count1{$_} < 2 } @array;
	#print $#array.":".$#unique."\n";
	# condense multiple items
	if($#array ne $#unique) {
		@duplicates = grep { $count2{$_} > 1 } grep { ++$count2{$_} > 1 } @array; 	# array of items that appear more than once
		@list = grep { ++$count3{$_} < 2 } @duplicates;					# make list unique
		@once = grep { $count4{$_} ==1 } grep { ++$count4{$_} < 2 } @array; 		# array of items that appear just once
		foreach my $list (@list) {
			$num = grep /^$list/i, @array;
			push(@once,"$num $list");	
			print "DUPE:$num $list\n";
		}
	} else {
		@once = @array;
		foreach (@once) {
			print "ONCE:$_\n";
		}
	}
		
	#$num_apple = grep /^apple$/i, @fruits;
	return (@once);
}

# returns the condition of an item
sub ret_condition {
	my $class = shift;
	my $num = shift;
	
	my %condition = (
		'0'		=>	"unknown",
		'1'		=>	"a complete wreck",
		'2'		=>	"atrocious",
		'3'		=>	"very poor",
		'4'		=>	"rather poor",
		'5'		=>	"poor",
		'6'		=>	"fairly poor",
		'7'		=>	"fairly good",
		'8'		=>	"decent",
		'9'		=>	"good",
		'10'		=>	"rather good",
		'11'		=>	"very good",
		'12'		=>	"excellent",
	);
 	#print "CONDITION:".$condition{1}."\n";
	return ($condition{$num});
}

sub more_than_one {

}


sub valid_direction
{
 my $self = shift;
 my $dir = shift; 

 my @validdirs = qw(north south east west);

 foreach my $d (@validdirs)
 {
  if ($d eq $dir) { return(1); } 
 }

 return(0); 

}




sub kernel {
	my $class = shift;
	if (my $k=shift) {$kernel = $k}
	else {return $kernel}
	return 0;
}


##########################################################################
# print 'leaves' messages to rooms

sub leaves_messages {
	my $class = shift;

	my %entries = %{Utility->reference("leaves")};
	my @entries = values %entries;
	my @runnow;
}


##########################################################################
# scheduler

# schedule('object' => $self, 'inseconds' => 15, action => "command($arg1, $arg2...)", priority = 1  );
sub schedule {
	my $class = shift;
        my %passed = @_;
	
	my $object        = $passed{object};
	my $name          = $passed{name};
	my $type          = $passed{type};
	my $inseconds     = $passed{inseconds};
	my $action        = $passed{action};
	my $priority      = $passed{priority};
	my $when_mudtime  = $passed{when_mudtime};
	my $when_realtime = $passed{when_realtime};
	
	my $entry;
	
	$scheduleid++;
	
	$entry->{id}       = $scheduleid;
	$entry->{object}   = $passed{object};
	$entry->{name}     = $passed{name};
	$entry->{type}     = $passed{type}; # e.g. alarm or spell
	$entry->{time}     = time() + $inseconds;
	$entry->{action}   = $passed{action};
	$entry->{priority} = $passed{priority};

	print "SCHEDULE ADD: $name,$object,$inseconds,$action\n";
	Utility->add("schedule",$scheduleid,$entry);	# store schedule entry
}

# delete all entries for a player/monster/carriage etc
# need to add a list in the player/monster/carriage modules, which has the ids of the schedule entries
sub delete_from_schedule {
	my $class = shift;
	my $deletename = shift;
	# sort through schedule to find entries to delete
	my %entries = %{Utility->reference("schedule")};
	my @entries = values %entries;
		
	foreach my $entry (@entries) {
		if($entry->{name} eq $deletename) {
			Utility->del("schedule",$entry->{id});	# delete schedule entry
		}
	}
}

sub run_scheduler {
	my $class = shift;

	print "RUNNING SCHEDULER\n";
	
	# sort through schedule to find entries to run this second or before and delete it from main schedule
	my %entries = %{Utility->reference("schedule")};
	my @entries = values %entries;
		
	foreach my $entry (@entries) {
 		print "ENTRY:".$entry->{time}.":".time().":".$entry->{time}.":".$entry->{action}."\n";
		if($entry->{time} <= time()) {
			#push(@runnow,$entry);
			if($entry->{action} =~ /::/) {
				my $run = eval { return $entry->{action}; };
				print "RUN1: $run\n";
				my $success = eval ($run);
			} else {
				my $object = $entry->{object};
				(my $action, my $args) = split(/\(/,$entry->{action});
				chop($args);
				my $success = eval { &{$object->$action}($object,$args); };
# 				my $run = eval { return "\$entry->{object}->".$entry->{action};};
# 				print "RUN2: $run\n";
# 				my $success = eval {$run};
				
			}
			Utility->del("schedule",$entry->{id});	# delete schedule entry
		}
	}

	# sort though these entries and rank them by priority
	
	
	
	# run through these entries executing them
	
}

# # scheduler
# it is a hook from the system heartbeat.
# 
# it accepts: 'object', 'inseconds', 'action', 'priority', 'when_mudtime', 'when_realtime'
# 
# if(!$object) then just run the ${$action}.
# 
# 'when_mudtime' and 'when_realtime' are translated into 'inseconds'
# 
# it has a schedule hash of 'time' => $entry
# 
# it sorts this hash by keys ascending, and the starts a foreach
# 
# if $_ = localtime, it puts it into a new hash %runnow and deletes it from the main hash
# when $_ != localtime, stop running through main hash, to save cpu
# 
# sort %runnow into execute order, of priorities 5 -> 0
# then foreach thru keys %runnow
# then run the $entry->action
# 
# done


sub log 
{
 my ($class,$severity,$source,$what) = @_;

 my $config = GameConfig->new();

 if ($config->{'LOGLEVEL'} >= $severity) 
 {
#  if (World->valid_object('controlroom'))
#  {
#   World->get_object('controlroom')->tell_room("The screen labeled '$source' updates with: $what");
#  }
#  print time . ": $source ($severity): $what\n";

  my $proc = $config->{'LOGNAME'};

  setlogsock('unix');
  openlog($proc, "ndelay,pid", 'daemon');

  syslog('info', "$source ($severity) $what");

  closelog();


 }
}


sub is_integer {
   my $self = shift;
   defined $_[0] && $_[0] =~ /^[+-]?\d+$/;
}


sub notify_entry 
{
 my $self = shift;
 my $locationid = shift;
 my $objectid = shift;

 my $enteringobj = Object->get_world_obj($objectid);

 foreach my $obj (Object->get_objects_in($locationid))
 {
  if ($obj->{attr}->{location} == $locationid)
  {
   if (defined $obj->{attr}->{session})
   {
    Utility->log(7,'utility',"sending object " . $obj->id() . " entry notify for: $objectid into $locationid");
    $kernel->call($obj->{attr}->{session}, "hear", $enteringobj->name() . " has arrived.");
   }
  }
 }
}


sub display_file
{
 my $self = shift;
 my $player = shift;
 my $filename = shift;

 if (open(FH,"<$filename"))
 {
  while (<FH>)
  {
   chomp();
  
   if ($_)
   {
    $player->tell_user($_);
   }
   else
   {
    $player->tell_user(" "); 
   }
  }
  close(FH);
 } 
 else
 {
  Utility->log(1,'utility',"Cannot open file '$filename' for display");
 }
}


sub array_to_string
{
 my $self = shift;
 my (@input) = @_; 
 
 my $input = '';

 foreach my $i (@input)
 {
  if ($input eq '')
  {
   $input .= $i;
  }
  else
  {
   $input .= " $i";
  }
 }

 return($input);
}


sub get_objectmap
{
 my $self = shift;
 my (@objs) = @_;

 my %nounmap;

 foreach my $obj (@objs)
 {
  #aliases
  foreach my $alias ($obj->get_aliases())
  {
   (%nounmap) = Utility->nounmap_add(\%nounmap,$obj,$alias);
  }

  #object name
  if (defined $obj->name())
  {
   (%nounmap) = Utility->nounmap_add(\%nounmap,$obj, lc($obj->name()));
  }

  #object color + type
  if (defined $obj->{attr}->{color})
  {
   (%nounmap) = Utility->nounmap_add(\%nounmap,$obj, $obj->{attr}->{color} . " " . lc($obj->type()));
  }

  #object type - not now
#  (%nounmap) = Utility->nounmap_add(\%nounmap,$obj, lc($obj->type()));

  #object id
  (%nounmap) = Utility->nounmap_add(\%nounmap,$obj, $obj->id());

  #object serial
  (%nounmap) = Utility->nounmap_add(\%nounmap,$obj, '#' . $obj->serial());
 }

 return(%nounmap);
}


sub nounmap_add
{
 my $self = shift;
 my $nounmapref = shift;
 my ($obj) = shift;
 my $noun = shift;

 my %nounmap = %$nounmapref;

 if ((defined $nounmap{$noun}) && ($nounmap{$noun} ne $obj))
 {
  $nounmap{$noun} = 'dup';
 }
 else
 {
  $nounmap{$noun} = $obj;
 }


 return(%nounmap);
}


sub noop
{
 return(1);
}

1;  # so the require or use succeeds

