package World;

use strict;
no strict 'refs';
use Carp;

use GameConfig;

use Commands;
use Messages;

use Object;
use Verb;
use Noun;
use Room;
use Door;
use Player;
use Item;
use Character;
use Zone;


my %world;


sub new
{
 my $self = shift;

 Utility->log(2,'world',"new world"); 

}


sub world_beat
{
 foreach my $objid (keys %world)
 {
  if (defined $world{$objid}->{attr}->{beat_event})
  {
   &{$world{$objid}->{attr}->{beat_event}}($world{$objid});
  }
 }

}


sub delete_object
{
 my $self = shift;
 my $id = shift;

 Utility->log(7,'world',"delete_object: $id");

# $world{$id} = {};

 delete $world{$id};
}

sub register_object
{
 my $self = shift;
 my $id = shift;
 my $object = shift;
 
 $world{$id} = $object;

 Utility->log(8,'world',"register_object: $id");
}


sub get_object
{
 my $self = shift;
 my $id = shift;

 if (defined $world{$id})
 {
  Utility->log(8,'world',"get_object: returned object '$id'");
  return($world{$id});
 }
 else
 {
  Utility->log(1,'world',"get_object: no such object '$id'");
 }

}


sub get_world_objects
{
 my $self = shift;

 return(%world);
}


sub get_objects_at
{
 my $self = shift;
 my $objid = shift;

 my @objects;

 foreach my $id (sort keys %world)
 {
  if ($world{$id}->location() eq $objid)
  {
   push(@objects,$world{$id});
#   foreach my $o (World->get_objects_at($world{$id}->id()))
#   {
#    push(@objects,$o);
#   }
  }
 }

 return(@objects);
}


sub get_phantoms_at
{
 my $self = shift;
 my $objid = shift;

 my @objects;

 foreach my $id (sort keys %world)
 {
  if ($world{$id}->is_phantom_at($objid))
  {

   push(@objects,$world{$id});
#   foreach my $o (World->get_objects_at($world{$id}->id()))
#   {
#    push(@objects,$o);
#   }
  }
 }

 return(@objects);
}


sub valid_object
{
 my $self = shift;
 my $id = shift;

 if (defined $world{$id})
 {
  Utility->log(9,'world',"valid_object: $id exists");
  return(1);
 }

 Utility->log(7,'world',"valid_object: $id does not exist");
 return(0);
}


sub load_world_objects
{
 my $self = shift;

 Utility->log(7,'world',"load_world_objects: loading objects...");
 
 &load_object_dir('.','objects');

 Utility->log(7,'world',"load_world_objects: done loading objects");
}



sub load_object_dir
{
 my ($path, $name) = @_;

 if ($name =~ m/.+\.po$/)
 {
  Utility->log(8,'world',"load_world_objects: loading $path/$name");

  if (open (FH, "$path/$name"))
  {
   undef $/;
   my $buffer = <FH>;
   $/ = "\n";
   close FH;

   my $obj = eval $buffer;

   if ($@ || !$obj)
   {
    Utility->log(1,'world',"load_world_objects: error loading $path/$name $@ $obj");
   }
  }
  else
  {
   Utility->log(1,'world',"load_world_objects: error opening $path/$name");
  }
 }

 return unless -d("$path/$name");
 local *DIRH;
 opendir DIRH, "$path/$name"
 or warn("couldn't open $path/$name: $!"), return;
 my @files = grep { $_ ne '.' and $_ ne '..' } readdir DIRH;
 closedir DIRH;
 for (@files) {
  load_object_dir("$path/$name", $_); 
 }
}


sub broadcast
{
 my $self = shift;
 my $txt = shift;
 my $obj = shift;

 my $objid = '';

 if (defined $obj)
 {
  $objid = $obj->id();
 }

 foreach my $o (keys %world)
 {
  if ($world{$o}->id() ne $objid)
  { 
   if (defined $world{$o}->{attr}->{session})
   {
    $world{$o}->tell_user($txt);
   }
  }
 }
}


sub get_environment
{
 my $self = shift;
 my $obj = shift;

 my @env;

 # inventory
 foreach my $o (World->get_objects_at($obj->id()))
 {
  if ($o->visible())
  {
   push(@env,$o);
  }
 }

 # commands - replaced by phantoms
# foreach my $o (World->get_objects_at('controlroom'))
# {
#  push(@env,$o);
# }

 
 # room contents (includes self and phantoms)
 foreach my $o (World->get_objects_at($obj->location()))
 {
  push(@env,$o);
  foreach my $oc (World->get_objects_at($o->id()))
  {
   if ($oc->visible())
   {
    push(@env,$oc);
   }
  } 
 }
 
 # phantoms

 foreach my $o (World->get_phantoms_at($obj->location()))
 {
  if ($o->visible())
  {
   push(@env,$o);
  }
 } 



 # room and doors
 my $o = World->get_object($obj->location());

 push(@env,$o);

 foreach my $exit (keys %{$o->{attr}->{exits}})
 {
  if (defined $o->{attr}->{exits}->{$exit}->{doorid}) 
  {
   if (World->get_object($o->{attr}->{exits}->{$exit}->{doorid})->visible())
   {
    push(@env,World->get_object($o->{attr}->{exits}->{$exit}->{doorid}));
   }
  }
 } 

 return(@env);
}


1;

