#!/usr/bin/perl


use Bit::Vector;

use constant {
        FLAG_E          => 7,
        FLAG_F          => 6,
        FLAG_H          => 5,
        FLAG_I          => 4,
        FLAG_N          => 3,
        FLAG_Z          => 2,
        FLAG_V          => 1,
        FLAG_C          => 0,
        IRQ_NORMAL      => 0,
        IRQ_SYNC        => 1,
        IRQ_CWAI        => 2
};



$a = Bit::Vector->new_Hex(8,'0');
$b = Bit::Vector->new_Hex(8,'0'); 

$dp = Bit::Vector->new_Hex(8,'0');
$cc = Bit::Vector->new_Hex(8,'0');

$pc = Bit::Vector->new_Hex(16,'0');

$x = Bit::Vector->new_Hex(16,'0');
$y = Bit::Vector->new_Hex(16,'0');

$u = Bit::Vector->new_Hex(16,'0');
$s = Bit::Vector->new_Hex(16,'0');

$ipc = Bit::Vector->new_Hex(16,'0');

$irq_start_time = 0;

$ea = 0;

$cpu_clk = 0;

$cpu_period = 0;

$cpu_quit = 1;

$irqs_pending = 0;

$firqs_pending = 0;

$cc_changed = 0;

$dump_cycles_on_success = 0;

$trace_enabled = 1;

$irq = 0;

$firq = 0;


#memory
@mem = ();

for (my $addr = 0;$addr < 65535;$addr++)
{
 push(@mem,Bit::Vector->new_Hex(8,0));
}





$offset = 0;
$initaddr = 0;


# load memory

$counter = $offset;

if (open (FH, "<$ARGV[0]"))
{
 undef $/;
 my $buffer = <FH>;
 $/ = "\n";
 close FH;

 for (my $addr=0;$addr<length($buffer);$addr++)
 {
  $mem[$counter] = Bit::Vector->new_Dec(8,ord(substr($buffer,$addr,1)));
  $counter++;
 }
}
else
{
 die "Can't open $ARGV[0]: $!\n";
}

print "\nLoaded $counter bytes in memory at $offset\n\n";


dump_dec(0,$counter);


dump_cc();

set_cc(FLAG_E,1);
set_cc(FLAG_Z,1);
set_cc(FLAG_C,1);

dump_cc();

print "stack test\n";

$sp = Bit::Vector->new_Dec(16,1000);

push16(Bit::Vector->new_Dec(16,1234));
push8(Bit::Vector->new_Dec(16,78));
push16(Bit::Vector->new_Dec(16,7654));
push8(Bit::Vector->new_Dec(16,56));

print pull8()->to_Dec;
print "\n";
print pull16()->to_Dec;
print "\n";
print pull8()->to_Dec; 
print "\n";
print pull16()->to_Dec;
print "\n";






#################################################################################

# mem 


sub dump_dec
{
 my ($start,$stop) = @_;

 for (my $addr = $start;$addr <= $stop;$addr = $addr + 32)
 {
  printf("%-5s", Bit::Vector->new_Dec(16,$addr)->to_Hex());
  for (my $addy = 0;$addy < 32;$addy++)
  {
#   print $mem->get($x + $y);
   print $mem[ $addr + $addy]->to_Hex();
   print " ";
  }
  print "\n";
 }
}





# cpu

sub get_cc
{
 my $flag = shift;

 return($cc->bit_test($flag));
}


sub set_cc
{
 my $flag = shift;
 my $val = shift;

 if ($val) 
 {
  $cc->Bit_On($flag);
 }
 else
 {
  $cc->Bit_Off($flag);
 }

}


sub dump_cc
{

 print "CC: ";

 if (get_cc(FLAG_E)) { print "E"; } else { print " "; }
 if (get_cc(FLAG_F)) { print "F"; } else { print " "; }
 if (get_cc(FLAG_H)) { print "H"; } else { print " "; }
 if (get_cc(FLAG_I)) { print "I"; } else { print " "; }
 if (get_cc(FLAG_N)) { print "N"; } else { print " "; }
 if (get_cc(FLAG_Z)) { print "Z"; } else { print " "; }
 if (get_cc(FLAG_V)) { print "V"; } else { print " "; }
 if (get_cc(FLAG_C)) { print "C"; } else { print " "; }

 print "\n";
}

sub w2b
{
 my $val = shift;

 return(Bit::Vector->new_Bin(8,substr($val->to_Bin(),0,8)),Bit::Vector->new_Bin(8,substr($val->to_Bin(),8,8)));
}

sub b2w
{
 my $hi = shift;
 my $lo = shift;

 return(Bit::Vector->new_Bin(16,$hi->to_Bin . $lo->to_Bin));
}


sub d
{
 my $val = shift;

 if (defined $val)
 {
  $a = Bit::Vector->new_Bin(8,substr($val->to_Bin(),0,8));
  $b = Bit::Vector->new_Bin(8,substr($val->to_Bin(),8,8));
 }
 
 return(Bit::Vector->new_Bin(16,$a->to_Bin() . $b->to_Bin()));
}



sub push8
{
 my $val = shift;
 
 $sp->decrement();
 $mem[$sp->to_Dec()] = $val;
}

sub pull8
{
 my $val = $mem[$sp->to_Dec()];
 $sp->increment();

 return($val->Clone());
}

sub push16
{
 my $val = shift;

 my ($hi,$lo) = w2b($val);

 push8($lo);
 push8($hi); 
}

sub pull16
{
 return(b2w(pull8,pull8));
}


sub pc_read8
{
 return($mem[$pc->to_Dec()]);
}

sub pc_read16
{
 return(b2w($mem[$pc->to_Dec()],$mem[$pc->to_Dec() + 1]));
}


sub sign_extend
{
 my $val = shift;
 
 $nval = $val->Clone();
 $nval->Resize(16); 

 return($nval);
}


sub ea_direct
{
 # ??
}


######################## subinsts

sub add16
{
 my $val1 = shift;
 my $val2 = shift;

 return(Bit::Vector->add($val1,$val2,0));

######################## instructions

sub abx
{
 $x = add16($b,$x);
}





1;

